//c 24/03/2022: use joystick
//d 26/03/2022: add click , wheel
//e 29/03/2022: on click piedzo led, or during wheel mode, piedzo on/,,,/off led, wheel / y 
//g 26/04/2022: use a box with four buttons
//h 12/05/2022: led on during long press, use ezButton
//k 06/06/2022: remove bip to reduce delay
//m 05/10/2022: add pitches.h, change threshold_min _max, delay and delta
//
//a 26/11/2022: rename to ms_pc_x, ms_pc_a merge ms and pc dev
//b 16/02/2023: joystick 3 types of 'move' and ezButtons for debounce
//c 18/03/2023: update to pin_out v2 ( left  = 7; right = 8; wheel = 9; press = 10; piedzo    = 13;)
//@fr add pc ms speed selection
//@fr test double click

#include <Mouse.h>   
#include "pitches.h"
#include <ezButton.h>
     
#define USE_JOYSTICK        1
#define USE_DEBUG_JOYSTICK  1

#define USE_MOUSE           1
#define USE_BIP_ON_BUTTON   0

// define button pin
const int pin_left  = 5;
const int pin_right = 4;
const int pin_wheel = 3;
const int pin_press = 2;

#define HW_V1

#ifdef HW_V1
const int pin_led_left  = 8; //red
const int pin_led_right = 10; //green
const int pin_led_wheel = 11; //yellow
const int pin_led_press = 12;//blue
const int pin_piedzo    = 9; //define piedzo pin
#endif

#ifdef HW_V2
const int pin_led_left  = 7; //white(G)
const int pin_led_right = 8; //white(D)
const int pin_led_wheel = 9; //red
const int pin_led_press = 10;//blue
const int pin_piedzo    = 13;//define piedzo pin
#endif

// define response delay of the mouse, in ms
const int responseDelay = 5;

// notes in the melody:
const int melody[] = {
  NOTE_C4, NOTE_G3, NOTE_G3, NOTE_A3, NOTE_G3, 0, NOTE_B3, NOTE_C4
};

// note durations: 4 = quarter note, 8 = eighth note, etc.:
const int noteDurations[] = { 4, 8, 8, 4, 4, 4, 4, 4 };

//define analog pin for joystick
#define Axe_X A0
#define Axe_Y A1
    
// define state buttons
bool bt_left  = LOW;
bool bt_right = LOW;
bool bt_wheel = LOW;
bool bt_press = LOW;

// set wheel button state
bool state_wheel = false;
bool state_press = false;
bool state_press_left = false;
bool state_press_right = false;

// parameters for reading the joystick:
const int center        = 1024 / 2;          // resting position value
const int threshold_min = center-40;         // min resting threshold
const int threshold_max = center+40;         // max resting threshold
int xMin=1024,xMax=0,yMin=1024,yMax=0;

ezButton button_left(pin_left);
ezButton button_right(pin_right);
ezButton button_wheel(pin_wheel);
ezButton button_press(pin_press);

//*********************************************

void buzzer_right(void)  {
#if USE_BIP_ON_BUTTON
      buzzer(4,5);
#endif
}

void buzzer_left(void)  {
#if USE_BIP_ON_BUTTON
      buzzer(1,2);
#endif
}

void buzzer(int nStart,int nStop)
{
      // iterate over the notes of the melody:
      for (int thisNote = nStart; thisNote < nStop; thisNote++) {
 
        int noteDuration = 1000 / noteDurations[thisNote];
        tone(pin_piedzo, melody[thisNote], noteDuration);

        int pauseBetweenNotes = noteDuration * 1.3;
        delay(pauseBetweenNotes);

        noTone(pin_piedzo);
      }
}

//*********************************************

int toogle_press(void) {
  int result=0;

  //when the button press is pressed, toogle between press/release state
  if (state_press==false) {
    digitalWrite(pin_led_press, HIGH);  // indicate via LED
    Serial.println("(button press)--------------> PRESS");
    #if USE_BIP_ON_BUTTON
        buzzer(4,5);
    #endif
    state_press=true;
  }
  else {
    digitalWrite(pin_led_press, LOW);  // stop LED
    Serial.println("(button press)--------------> RELEASE");
    #if USE_BIP_ON_BUTTON
      buzzer(7,8);
    #endif
    state_press=false;
  }
  result=state_press;
  return result;
}

int toogle_wheel(void){
  int result=0;

  //when the wheel button is pressed, toogle between xy/wheel state
  if (state_wheel==false) {
    digitalWrite(pin_led_wheel, HIGH);  // indicate via LED
    Serial.println("(xy/wheel)--------------> MODE WHEEL");
    #if USE_BIP_ON_BUTTON
      buzzer(0,5);
    #endif
    state_wheel=true;
  }
  else {
    digitalWrite(pin_led_wheel, LOW);  // stop LED
    Serial.println("(xy/wheel)--------------> MODE XY");
    #if USE_BIP_ON_BUTTON
      buzzer(6,8);
    #endif     
    state_wheel=false;
  }
  result=state_wheel;
  return result;
}

//*********************************************

bool  action_button_left(){
  int result=0;

  if (state_press==false) {
    Serial.println("(button left)--------------> click ");
  
    digitalWrite(pin_led_left, HIGH);
    buzzer_left();
    delay(100);
    digitalWrite(pin_led_left, LOW);
    
    #if USE_MOUSE
      Mouse.click(MOUSE_LEFT);
    #endif
  }
  else
  {
    Serial.println("(button left)--------------> in press mode");
    if ((state_press_left==false) && (state_press_right==false)) {
      Serial.println("(button left)--------------> send press");
      digitalWrite(pin_led_left, HIGH);
      #if USE_MOUSE
        Mouse.press(MOUSE_LEFT);
      #endif
      state_press_left=true;
      state_press_right=false;
    }
    else if (state_press_left==true) {
      Serial.println("(button left)--------------> send release");
      digitalWrite(pin_led_left, LOW);
      #if USE_MOUSE
        Mouse.release(MOUSE_LEFT);
      #endif
      digitalWrite(pin_led_press, LOW);  // stop LED
      state_press_left=false;
      state_press_right=false;
      toogle_press();
    }
  }
  
  return result;
}


bool action_button_right(){
  int result=0;

  if (state_press==false) {
    Serial.println("(button right)-------------> click ");
  
    digitalWrite(pin_led_right, HIGH);
    buzzer_right();
    delay(100);
    digitalWrite(pin_led_right, LOW);
    
    #if USE_MOUSE
      Mouse.click(MOUSE_RIGHT);
    #endif
  }
  else
  {
    Serial.println("(button right)---------------> in press mode");
    
    if ((state_press_left==false) && (state_press_right==false)) {
      Serial.println("(button right)--------------> send press");
      digitalWrite(pin_led_right, HIGH);
      digitalWrite(pin_led_press, HIGH);
      #if USE_MOUSE
        Mouse.press(MOUSE_RIGHT);
      #endif
      state_press_left=false;
      state_press_right=true;
    }
    else if (state_press_right==true) {
      Serial.println("(button right)--------------> send release");
      #if USE_MOUSE
        Mouse.release(MOUSE_RIGHT);
      #endif
      digitalWrite(pin_led_right, LOW);
      digitalWrite(pin_led_press, LOW);
      state_press_left=false;
      state_press_right=false;
      toogle_press();
    }
  }
  
  return result;
}

//*********************************************

int readAxis(int thisAxis) {

  // read the analog input:
  unsigned int reading = analogRead(thisAxis);

  reading = 1023 - reading;
  
  // return the reading for this axis value:
  return reading;
}

signed char getMove(int axisPosition) {
  
  signed char ret = 0;
  int delta;
  const int delta_max=10;

  if ((axisPosition > threshold_min)&&(axisPosition < threshold_max))  
  {
    ret = 0; // inside min max > no move
  } 
  else 
  {
    if (axisPosition>threshold_max)     
    {
      delta=(axisPosition-threshold_max)/40;
      if (delta<delta_max){ ret = delta; } else { ret = delta_max; }
    }
    else 
    {
      if (axisPosition<threshold_min)
      {
        delta=(threshold_min-axisPosition)/40;           
        if (delta<delta_max){ ret = -delta; } else { ret = -delta_max; }
      }
    }    
  }
  return ret;
}

//*********************************************

void setup() {
  Serial.begin(9600);
  //while(!Serial)     ;
    
  //pinMode(pin_left,  INPUT_PULLUP);

  button_left.setDebounceTime(200);
  button_right.setDebounceTime(100);
  button_wheel.setDebounceTime(50);
  button_press.setDebounceTime(50);

 
  // initialize the LED pin as an output:
  pinMode(pin_led_left,  OUTPUT);
  pinMode(pin_led_right, OUTPUT);
  pinMode(pin_led_wheel, OUTPUT);
  pinMode(pin_led_press, OUTPUT);
  
  // use A0 A1 analog pin for joystick axis
  
  // setup default state
  state_wheel = false;
  state_press = false;
    
  //take control of the mouse:
  #if USE_MOUSE
    Serial.println("Mouse functions enable for 32u4 or SAMD");
    Mouse.begin();
  #endif

}

//*********************************************

void loop() {
  
  // *** polling mode : ***

  button_left.loop();
  button_right.loop();
  button_wheel.loop();
  button_press.loop();

  //  Serial.println(button_wheel.getState());
  //when the wheel button is pressed, toogle between xy/wheel state 
  if(button_wheel.isPressed()) {
    Serial.println("(button wheel)-------------->");
    toogle_wheel();
  }

  if (state_wheel==false) {
    //Serial.println(state_wheel);
    if(button_press.isPressed())
      toogle_press();
      //when the press button is pressed, toogle between click/long press state
        
    if(button_left.isPressed())
      action_button_left();
  
    if(button_right.isPressed())
      action_button_right(); 
  }
  else {
  
  }
  
  //read & display axe x&y value
  #if USE_JOYSTICK

    // read the two axes:
    int xReading = readAxis(Axe_X);
    int yReading = readAxis(Axe_Y);

    // scale the two axes:
    int xScale = getMove(xReading);
    int yScale = getMove(yReading);

    if (abs(xScale)<10)  {delay(100-abs(xScale)*10);  }
    
    // update x y, min max:
    if (xReading<xMin) {xMin=xReading;}
    if (xReading>xMax) {xMax=xReading;}
    
    if (yReading<yMin) {yMin=yReading;}
    if (yReading>yMax) {yMax=yReading;}
    
    #if USE_DEBUG_JOYSTICK
    
    Serial.print(xMin);
    Serial.print(" ");
    Serial.print(xReading);
    Serial.print(" ");   
    Serial.print(xMax);
    Serial.print(" ");
    
    Serial.print(yMin);
    Serial.print(" ");
    Serial.print(yReading);
    Serial.print(" ");
    Serial.print(yMax);
    Serial.println(" ");
    
    #endif 
    
    #if USE_MOUSE
      if (state_wheel) {
        
        //use y as wheel
        if (yScale>0) { Mouse.move(0,0, 1);} else
        if (yScale<0) { Mouse.move(0,0, -1);}

        delay(50);
      }
      else {
        if ((xScale != 0)||(yScale!=0)) {
          Mouse.move(xScale, yScale, 0);
        }
      }
    #endif
  
  #endif
    
  delay(responseDelay);
}
